/*
 * File         : Acceptor.h
 * Author       : Tomasz Stachowiak
 * Created      : Fri May 15 18:22:19 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#ifndef __ACCEPTOR_H
#define __ACCEPTOR_H

#include <BVN/BVN.h>
#include <BVN/Event_Handler.h>
#include <BVN/Reactor.h>

template <class SVC_HANDLER, class PEER_ACCEPTOR>
class Acceptor : public Event_Handler
{
 public:

  Acceptor();

  virtual ~Acceptor();

  virtual int open(PEER_ACCEPTOR::PEER_ADDR *local_addr,
		   Reactor *reactor);

  virtual int handle_event(Event_Type et);

  virtual HANDLE get_handle(void);

  virtual HANDLE handle_close(void);

 private:

  PEER_ACCEPTOR peer_acceptor_;
  Reactor *reactor_;
};





#define SH SVC_HANDLER
#define PA PEER_ACCEPTOR

template <class SH, class PA>
Acceptor<SH, PA>::Acceptor()
{
}

template <class SH, class PA>
Acceptor<SH, PA>::~Acceptor()
{
}

template <class SH, class PA>
int Acceptor<SH, PA>::open(PEER_ACCEPTOR::PEER_ADDR *local_addr,
		   Reactor *reactor)
{
  this->reactor_ = reactor;
  if(this->peer_acceptor_.open(local_addr) == -1)
    {
      BVN_ERROR("Acceptor: open\n");
      return -1;
    }

  if(this->reactor_->register_handler(this, READ_EVENT) == -1)
    {
      BVN_ERROR("Acceptor: open\n");
      return -1;
    }

  return 0;
}

template <class SH, class PA>
int Acceptor<SH, PA>::get_handle(void)
{
  return this->peer_acceptor_.get_handle();
}

template <class SH, class PA>
int Acceptor<SH, PA>::handle_event(Event_Type et)
{

  BVN_DEBUG("Acceptor::handle_event\n");

  SH *srv_handler = new SH;
  if(srv_handler == NULL)
    {
      BVN_ERROR("Acceptor: handle_event\n");
      return -1;
    }

  if(this->peer_acceptor_.accept(srv_handler->peer()) == -1)
    {
      delete srv_handler;
      BVN_ERROR("Acceptor: handle_event\n");
      return -1;
    }
  
  if(srv_handler->open(reactor_) == -1)
    {
      BVN_ERROR("Acceptor: handle_event\n");
      return -1;
    }

  return 0;
}

template <class SH, class PA>
int Acceptor<SH, PA>::handle_close()
{
  return this->peer_acceptor_.close();
}

#endif

